@@ -11,6 +11,8 @@ from base import NodeModel
 
 from neo4jrestclient.constants import RELATIONSHIPS_ALL, RELATIONSHIPS_IN, RELATIONSHIPS_OUT
 
+from collections import defaultdict
+
 class LazyModel(object):
     """
     A proxy class that enables relationships to have str targets, eg
@@ -18,10 +20,10 @@ class LazyModel(object):
         actors = Relationship('Actor', ...)
     """
     def __init__(self, cls, field, name, setup_reversed):
-        add_lazy_relation(cls, field, name, self.__setup)
         self.__setup_reversed = setup_reversed
+        add_lazy_relation(cls, field, name, self.__setup)
     def __setup(self, field, target, source):
-        if not issubclass(target, NodeModel)    :
+        if not issubclass(target, NodeModel):
             raise TypeError("Relationships may only extend from Nodes.")
         self.__target = target
         self.__setup_reversed(target)
@@ -165,11 +167,7 @@ class Relationship(object):
         if not issubclass(source, NodeModel):
             raise TypeError("Relationships may only extend from Nodes.")
         self.creation_counter = source.creation_counter
-        if isinstance(self.__target, basestring):
-            target = LazyModel(source, self, self.__target,
-                        lambda target: bound._setup_reversed(target))
-        else:
-            target = self.__target
+        
         if hasattr(self, 'Model'):
             if self.__single:
                 Bound = SingleRelationship
@@ -185,6 +183,10 @@ class Relationship(object):
             bound = Bound(self, source, self.__name or name, name)
         source._meta.add_field(bound)
         setattr(source, name, bound)
+        if isinstance(self.__target, basestring):
+            self.__target = LazyModel(source, self, self.__target,
+                        lambda target: bound._setup_reversed(target))
+        target = self.__target
         if not self.__is_reversed:
             bound._setup_reversed(target)
 
@@ -441,24 +443,18 @@ class MultipleNodes(BoundRelationship):
         if value is not None:
             if state.get(self.name) is None:
                 state[self.name] = RelationshipInstance(self, obj)
+            items = list(state[self.name].all())
+            notsaved = state[self.name]._added
+            if items and len(notsaved) < len(items):
+                state[self.name].remove(*items) 
+                #TODO: make it so it only removes authors not in value
+                #      and remove authors already there from value
+                #XXX: only works when removing from saved nodes
+            if notsaved:
+                notsaved[:] = [] #TODO give rel instance a method for this?
             if hasattr(value, '__iter__'):
-                items = list(state[self.name].all())
-                notsaved = state[self.name]._RelationshipInstance__added
-                if items and len(notsaved) < len(items):
-                    state[self.name].remove(*items) 
-                    #TODO: make it so it only removes authors not in value
-                    #      and remove authors already there from value
-                    #XXX: only works when removing from saved nodes
-                if notsaved:
-                    notsaved[:] = []
                 state[self.name].add(*value)
             else:
-                items = list(state[self.name].all())
-                notsaved = state[self.name]._RelationshipInstance__added
-                if items and len(notsaved) < len(items):
-                    state[self.name].remove(*items)
-                if notsaved:
-                    notsaved[:] = []
                 state[self.name].add(value)
 
     def value_to_string(self, obj):
@@ -508,11 +504,6 @@ class MultipleNodes(BoundRelationship):
         return super(MultipleNodes, self). \
                     _create_neo_relationship(node, *args, **kwargs)
 
-    def _extract_relationship(self, obj, ordered=False):
-        rels = self._load_relationships(obj.node, ordered=ordered)
-        #yes, we just choose one without further info (first if ordered)
-        return rels[0]
-
 class MultipleRelationships(BoundRelationshipModel): # WAIT!
     @not_implemented
     def _get_relationship(self, obj, state):
@@ -525,29 +516,34 @@ class RelationshipInstance(models.Manager):
     def __init__(self, rel, obj):
         self.__rel = rel
         self.__obj = obj
-        self.__added = [] # contains domain objects
-        self.__removed = set() # contains relationships
+        self._added = [] # contains domain objects
+        self._removed = set() # contains relationships
 
     ordered = property(lambda self: self.__rel.ordered)
 
     def __save__(self, node):
         #Deletes all relationships removed since last save and adds any new
         #relatonships to the database.
-        for relationship in self.__removed:
+        for relationship in self._removed:
             relationship.delete()
-        for obj in self.__added:
+        for obj in self._added:
             self.__rel._create_neo_relationship(node, obj)
-        self.__removed.clear()
-        self.__added[:] = []
+        self._removed.clear()
+        self._added[:] = []
 
     def _neo4j_relationships(self, node):
         for rel in self.__rel._load_relationships(node, ordered=self.ordered):
-            if rel not in self.__removed:
+            if rel not in self._removed:
                 yield rel
 
     @property
     def _new(self):
-        for item in self.__added:
+        for item in self._added:
+            yield item
+
+    @property
+    def _old(self):
+        for item in self._removed:
             yield item
 
     def add(self, *objs):
@@ -557,7 +553,7 @@ class RelationshipInstance(models.Manager):
         """
         for obj in objs:
             self.__rel.accept(obj)
-        self.__added.extend(objs)
+        self._added.extend(objs)
 
     def remove(self, *objs):
         """
@@ -565,14 +561,39 @@ class RelationshipInstance(models.Manager):
         remove the first relationship to this object- otherwise, remove one
         of the relationships indiscriminately.
         """
-        for obj in objs:
-            rel = self.__rel
-            self.__removed.add(rel._extract_relationship(obj,
-                                                         ordered=self.ordered))
+        rel = self.__rel
+        if hasattr(self.__obj, 'node'):
+            neo_rels = list(rel._load_relationships(self.__obj.node,
+                                                    ordered=self.ordered))
+            rels_by_node = defaultdict(list)
+            for neo_rel in neo_rels:
+                other_end = neo_rel.start if neo_rel.end == self.__obj.node\
+                            else neo_rel.end
+                rels_by_node[other_end].append(neo_rel)
+            nodes_to_remove = [o.node for o in objs if hasattr(o, 'node')]
+            unsaved_obj_to_remove = [o for o in objs if not hasattr(o, 'node')]
+            for obj in objs:
+                candidate_rels = rels_by_node[obj.node]\
+                        if hasattr(o, 'node') else []
+                if candidate_rels:
+                    self._removed.add(candidate_rels.pop(0))
+                else:
+                    try:
+                        self._added.remove(obj)
+                    except ValueError:
+                        raise rel.target_model.DoesNotExist("%r is not related to %r." % (obj, self.__obj))
+        else:
+            for obj in objs:
+                if obj in self._added:
+                    try:
+                        self._added.remove(obj)
+                    except ValueError:
+                        raise rel.target_model.DoesNotExist("%r is not related to %r." % (obj, self.__obj))
+
 
-    @not_implemented
     def clear(self):
-        pass
+        all_objs = list(self.all())
+        self.remove(*all_objs)
 
     @not_implemented
     def create(self, *args, **kwargs):
@@ -597,17 +618,20 @@ class RelationshipQuerySet(object):
                 yield rel
 
     def __iter__(self):
+        removed = list(self.__inst._old)
+        added = list(self.__inst._new)
         try:
             node = self.__obj.node
         except:
             pass
         else:
             buffered = buffer_iterator(
-                lambda rel: self.__rel._neo4j_instance(node, rel),
+                lambda rel: (rel, self.__rel._neo4j_instance(node, rel)),
                 self.__relationships(node), size=10)
-            for item in buffered:
-                yield item
-        for item in self.__inst._new:
+            for rel, item in buffered:
+                if rel not in removed:
+                    yield item
+        for item in added:
             if self.__keep_instance(item):
                 yield item
 
